<?php

namespace Drupal\commerce_stripe_payment_request_button\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_payment\PaymentMethodTypeManager;
use Drupal\commerce_payment\PaymentTypeManager;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Symfony\Component\HttpFoundation\Request;


/**
 * Provides Stripe Payment Request Button offsite payment.
 *
 * @CommercePaymentGateway(
 *   id = "commerce_stripe_payment_request_button",
 *   label = @Translation("Stripe Payment Request Button"),
 *   display_label = @Translation("Stripe Payment Request Button"),
 *    forms = {
 *     "offsite-payment" = "Drupal\commerce_stripe_payment_request_button\PluginForm\PaymentMethodOffsiteForm",
 *     "add-payment-method" = "Drupal\commerce_stripe_payment_request_button\PluginForm\PaymentMethodAddForm",
 *   },
 *   payment_method_types = {"credit_card"},
 *   credit_card_types = {
 *     "amex", "discover", "mastercard", "visa",
 *   },
 *   js_library = "commerce_stripe_payment_request_button/form",
 * )
 */
class PaymentRequestButton extends OffsitePaymentGatewayBase  {


  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, PaymentTypeManager $payment_type_manager, PaymentMethodTypeManager $payment_method_type_manager, TimeInterface $time) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $payment_type_manager, $payment_method_type_manager, $time);

    \Stripe\Stripe::setApiKey($this->configuration['secret_key']);
  }

  /**
   * {@inheritdoc}
   */
  public function getPublishableKey() {
    return $this->configuration['publishable_key'];
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
        'publishable_key' => '',
        'secret_key' => '',
        'api_logging' => '',
      ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['publishable_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Publishable Key'),
      '#default_value' => $this->configuration['publishable_key'],
      '#required' => TRUE,
    ];
    $form['secret_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Secret Key'),
      '#default_value' => $this->configuration['secret_key'],
      '#required' => TRUE,
    ];

    $form['api_logging'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Log webhook messages onNotify for debugging'),
      '#default_value' => $this->configuration['api_logging'],
      '#required' => FALSE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::validateConfigurationForm($form, $form_state);

    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);
      // Validate the secret key.
      $expected_livemode = $values['mode'] == 'live' ? TRUE : FALSE;
      if (!empty($values['secret_key'])) {
        try {
          \Stripe\Stripe::setApiKey($values['secret_key']);
          // Make sure we use the right mode for the secret keys.
          if (\Stripe\Balance::retrieve()->offsetGet('livemode') != $expected_livemode) {
            $form_state->setError($form['secret_key'], $this->t('The provided secret key is not for the selected mode (@mode).', ['@mode' => $values['mode']]));
          }
        }
        catch (\Stripe\Error\Base $e) {
          $form_state->setError($form['secret_key'], $this->t('Invalid secret key.'));
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);

    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);
      $this->configuration['publishable_key'] = $values['publishable_key'];
      $this->configuration['secret_key'] = $values['secret_key'];
      $this->configuration['api_logging'] = $values['api_logging'];
    }
  }

  public function onNotify(Request $request) {

    // Log the response message if request logging is enabled.
    if (!empty($this->configuration['api_logging'])) {
      \Drupal::logger('commerce_stripe_payment_request_button')
        ->debug('e-Commerce notification: <pre>@body</pre>', [
          '@body' => var_export($request->query->all(), TRUE),
        ]);
    }

    // Get Webhook request data.
    $webhook_data = $this->getRequestDataArray($request->getContent());
    //TODO get this data somehow and then act on it, make refund or order update

  }


  /**
   * Make single transaction.
   *
   * @throws \Stripe\Error\Base
   */
  public function makeTransaction() {

    // Parse the data from $_POST.
    $post = json_decode(file_get_contents('php://input'), TRUE);

    // Form values are passed as a serialized string.
    $form_values = [];
    parse_str($post['data']['serialized_form'], $form_values);


    // Init Stripe.
    $stripe_secret_key = $payment_method['settings']['stripe_secret_key'];
    \Stripe\Stripe::setApiKey($stripe_secret_key);


    $order = $this->order;
    $context = $this->context;

    $order_wrapper = entity_metadata_wrapper('commerce_order', $this->order);
    $order_total_data = $order_wrapper->commerce_order_total->value();
    $payment_method_id = $context['payment_method']['instance_id'];

    try {
      // Charge the user's card immediately.
      $charge = \Stripe\Charge::create([
        // NOTE: Stripe uses the same amount encoding as Drupal
        // Commerce (1050 = 10.50).
        'amount' => $order_total_data['amount'],
        'currency' => $order_total_data['currency_code'],
        'description' => $context['payment_request']['total']['label'],
        'metadata' => $context['metadata'],
        'source' => $context['stripe_data']['token']['id'],
      ]);

      // Prepare a transaction object to log the API response.
      $transaction = commerce_payment_transaction_new('commerce_stripe_payment_request', $order->order_id);
      $transaction->instance_id = $payment_method_id;
      $transaction->amount = $order_total_data['amount'];
      $transaction->currency_code = $order_total_data['currency_code'];
      $transaction->payload[REQUEST_TIME] = $charge->getLastResponse()->json;
      $transaction->remote_id = $charge->id;
      $transaction->remote_status = $charge->status;
      $transaction->status = COMMERCE_PAYMENT_STATUS_SUCCESS;
      $transaction->message = 'Stripe charge created successfully.';
      commerce_payment_transaction_save($transaction);
    }
    catch (\Stripe\Error\Base $e) {

      $transaction = commerce_payment_transaction_new('commerce_stripe_payment_request', $order->order_id);
      $transaction->instance_id = $payment_method_id;
      $transaction->amount = $order_total_data['amount'];
      $transaction->currency_code = $order_total_data['currency_code'];
      $transaction->payload[REQUEST_TIME] = $e->getJsonBody();
      $transaction->remote_status = 'error';
      $transaction->status = COMMERCE_PAYMENT_STATUS_FAILURE;
      $transaction->message = 'Could not complete Stripe charge. ' . $e->getMessage();
      commerce_payment_transaction_save($transaction);

      watchdog_exception('commerce_stripe_payment_request', $e);

      // Server will return 500 to the frontend.
      drupal_set_message(t('An error occurred processing this payment. The error is: @e', [
        '@e' => $e,
      ]), 'error');
    }

    commerce_order_status_update($this->order, 'checkout_complete', FALSE, FALSE);
    commerce_checkout_complete($this->order);

    $response = [
      'redirect_uri' => url(commerce_checkout_order_uri($this->order)),
    ];

    return $response;
    // below used in JS to redirect to thank you page at window.location.href = data.redirect_uri;
    //drupal_json_output($response);
  }


  /**
   * Sets the API key after the plugin is unserialized.
   */
  public function __wakeup() {
    \Stripe\Stripe::setApiKey($this->configuration['secret_key']);
  }

}